class Paging(object):

    def __init__(self, paging_data):
        self.paging_data = paging_data

    def has_next(self):
        if int(self.paging_data['page']) == int(self.paging_data['pages']):
            return False
        else:
            return True

    def has_previous(self):
        if int(self.paging_data['page']) == 1:
            return False
        else:
            return True

    def next(self):
        if self.has_next():
            return int(self.paging_data['page'])+1
        else:
            return False

    def previous(self):
        if self.has_previous():
            return int(self.paging_data['page'])-1
        else:
            return False



from marinetrafficapi.client import Client as MarineTrafficApi
from marinetrafficapi.exceptions import (MarineTrafficRequestApiException,
                                         MarineTrafficClientApiException,
                                         MarineTrafficFormatterException)



import click
import pprint


STATUS_OK = 'ok'
STATUS_ERROR = 'error'


class Debug(object):

    def __init__(self, client):
        self.client = client
        self.recorded_data = {}

    def __enter__(self):
        return self

    def __exit__(self, type, value, traceback):
        if self.client.debug:
            self.standard_output()

    def ok(self, key, value):
        self.record(key, value, STATUS_OK)

    def error(self, key, value):
        self.record(key, value, STATUS_ERROR)

    def record(self, key, value, status):
        if self.client.debug:
            self.recorded_data[key] = {'value': value, 'status': status}

    def show(self, standard_output=False):
        if standard_output:
            self.standard_output()
        else:
            return self.recorded_data

    def standard_output(self):
        click.echo('')
        click.echo(click.style('Marine Traffic API Client Debug', fg='green'))
        click.echo(click.style('-'*100, fg='green'))

        for key, value in self.recorded_data.items():

            value = value['value']
            if type(value) == object:
                value = dir(value)

            output_str = '%s: [%s] %s' % (click.style(str(key), fg='green'),
                                          click.style(str(type(value)), fg='blue'),
                                          pprint.pformat(value))

            click.echo(output_str)
            click.echo(click.style('-'*100, fg='green'))

        click.echo('')



from marinetrafficapi.query_params import QueryParams


class PS02QueryParams(QueryParams):
    """Query params params for PS02 API call."""

    params = {
        # The maximum age, in minutes, of the returned positions.
        # Maximum value for terrestrial coverage is 60.
        # Maximum value for satellite coverage is 180.
        'time_span': 'timespan',

        # Data filter: Vessel type.
        # (2=Fishing / 4=High Speed Craft / 6=Passenger / 7=Cargo / 8=Tanker)
        'ship_type': 'shiptype',
    }



from marinetrafficapi.bind import bind_request

from marinetrafficapi.vessels_positions.\
    PS01_vessel_historical_track.models import VesselHistoricalPosition
from marinetrafficapi.vessels_positions.\
    PS02_vessel_positions_of_a_static_fleet.models import StaticFleetVesselPosition

from marinetrafficapi.vessels_positions.\
    PS01_vessel_historical_track.query_params import PS01QueryParams
from marinetrafficapi.vessels_positions.\
    PS02_vessel_positions_of_a_static_fleet.query_params import PS02QueryParams

from marinetrafficapi.vessels_positions.\
    PS03_vessel_position_of_a_dynamic_fleet.query_params import PS03QueryParams
from marinetrafficapi.vessels_positions.\
    PS03_vessel_position_of_a_dynamic_fleet.models import DynamicFleetVesselPosition


class VesselPositions:
    """Retrieve forecasted information for any vessel.
    Get ETA and voyage related information using one of these APIs."""

    # PS01 - Get all historical positions for one or more vessels over a period of time.
    vessel_historical_track = bind_request(
        api_path='/exportvesseltrack',
        model=VesselHistoricalPosition,
        query_parameters=PS01QueryParams,
        default_parameters={
            'v': '2',
            'msgtype': 'simple',
            'protocol': 'jsono'
        }
    )

    # PS02 - Get positional information for a set of predefined vessels.
    static_fleet_vessel_positions = bind_request(
        api_path='/exportvessels',
        model=StaticFleetVesselPosition,
        query_parameters=PS02QueryParams,
        default_parameters={
            'v': '8',
            'msgtype': 'simple',
            'protocol': 'jsono'
        }
    )

    # PS03 - Monitor vessel activity for your MarineTraffic fleet(s)
    dynamic_fleet_vessel_positions = bind_request(
        api_path='/exportvessels',
        model=DynamicFleetVesselPosition,
        query_parameters=PS03QueryParams,
        default_parameters={
            'v': '8',
            'msgtype': 'simple',
            'protocol': 'jsono'
        }
    )



from marinetrafficapi.models import Model
from marinetrafficapi.fields import NumberField, RealNumberField, DatetimeField, TextField


class DynamicFleetVesselPosition(Model):
    """Get positional information for a set of predefined vessels."""

    mmsi = NumberField(index='MMSI',
                       desc="Maritime Mobile Service Identity - a nine-digit number "
                            "sent in digital form over a radio frequency that identifies "
                            "the vessel's transmitter station")

    imo = NumberField(index='IMO',
                      desc="International Maritime Organisation number - a "
                           "seven-digit number that uniquely identifies vessels")

    ship_id = NumberField(index='SHIP_ID',
                          desc="A uniquely assigned ID by MarineTraffic "
                               "for the subject vessel")

    longitude = RealNumberField(index='LON',
                                desc="A geographic coordinate that specifies the "
                                     "east-west position of the vessel on the "
                                     "Earth's surface")

    latitude = RealNumberField(index='LAT',
                               desc="a geographic coordinate that specifies the "
                                    "north-south position of the vessel on the "
                                    "Earth's surface")

    speed = NumberField(index='SPEED',
                        desc="The speed (in knots x10) that the subject vessel is "
                             "reporting according to AIS transmissions")

    heading = NumberField(index='HEADING',
                          desc="The heading (in degrees) that the subject vessel is "
                               "reporting according to AIS transmissions")

    status = NumberField(index='STATUS',
                         desc="The AIS Navigational Status of the subject vessel as "
                              "input by the vessel's crew - more. There might be "
                              "discrepancies with the vessel's detail page when vessel "
                              "speed is near zero (0) knots.")

    course = NumberField(index='COURSE',
                         desc="The course (in degrees) that the subject vessel is "
                              "reporting according to AIS transmissions")

    timestamp = DatetimeField(index='TIMESTAMP',
                              desc="The date and time (in UTC) that the subject vessel's "
                                   "position was recorded by MarineTraffic",
                              format='%Y-%m-%dT%H:%M:%S')

    dsrc = TextField(index='DSRC',
                     desc="Data Source - Defines whether the transmitted AIS data was "
                          "received by a Terrestrial or a Satellite AIS Station")

    utc_seconds = NumberField(index='UTC_SECONDS',
                              desc="The time slot that the subject vessel uses "
                                   "to transmit information")

    ship_name = TextField(index='SHIPNAME',
                          desc="The Shipname of the subject vessel")

    ship_type = NumberField(index='SHIPTYPE',
                            desc="The Shiptype of the subject vessel "
                                 "according to AIS transmissions")

    call_sign = TextField(index='CALLSIGN',
                          desc="A uniquely designated identifier for "
                               "the vessel's transmitter station")

    flag = TextField(index='FLAG',
                     desc="The flag of the subject vessel according to AIS transmissions")

    length = RealNumberField(index='LENGTH',
                             desc="The overall Length (in metres) of the subject vessel")

    width = RealNumberField(index='WIDTH',
                            desc="The Breadth (in metres) of the subject vessel")

    grt = NumberField(index='GRT',
                      desc="Gross Tonnage - unitless measure that calculates the "
                           "moulded volume of all enclosed spaces of a ship")

    dwt = NumberField(index='DWT',
                      desc="Deadweight - a measure (in metric tons) of how much weight a "
                           "vessel can safely carry (excluding the vessel's own weight")

    draught = NumberField(index='DRAUGHT',
                          desc="The Draught (in metres x10) of the subject vessel "
                               "according to the AIS transmissions")

    year_built = NumberField(index='YEAR_BUILT',
                             desc="The year that the subject vessel was built")

    rot = NumberField(index='ROT',
                      desc="Rate of Turn")

    type_name = TextField(index='TYPE_NAME',
                          desc="The Type of the subject vessel")

    ais_type_summary = TextField(index='AIS_TYPE_SUMMARY',
                                 desc="Further explanation of the SHIPTYPE ID")

    destination = TextField(index='DESTINATION',
                            desc="The Destination of the subject vessel "
                                 "according to the AIS transmissions")

    eta = DatetimeField(index='ETA',
                        desc="The Estimated Time of Arrival to Destination of the "
                             "subject vessel according to the AIS transmissions",
                        format='%Y-%m-%dT%H:%M:%S')

    current_port = TextField(index='CURRENT_PORT',
                             desc="The name of the Port the subject vessel is "
                                  "currently in (NULL if the vessel is underway)")

    last_port = TextField(index='LAST_PORT',
                          desc="The Name of the Last Port the vessel has visited")

    last_port_time = DatetimeField(index='LAST_PORT_TIME',
                                   desc="The Date and Time (in UTC) that the "
                                        "subject vessel departed from the Last Port",
                                   format='%Y-%m-%dT%H:%M:%S')

    current_port_id = NumberField(index='CURRENT_PORT_ID',
                                  desc="A uniquely assigned ID by "
                                       "MarineTraffic for the Current Port")

    current_port_unlocode = TextField(index='CURRENT_PORT_UNLOCODE',
                                      desc="A uniquely assigned ID by "
                                           "United Nations for the Current Port")

    current_port_country = TextField(index='CURRENT_PORT_COUNTRY',
                                     desc="The Country that the Current Port is located at")

    last_port_id = NumberField(index='LAST_PORT_ID',
                               desc="A uniquely assigned ID by MarineTraffic for the Last Port")

    last_port_unlocode = TextField(index='LAST_PORT_UNLOCODE',
                                   desc="A uniquely assigned ID by "
                                        "United Nations for the Last Port")

    last_port_country = TextField(index='LAST_PORT_COUNTRY',
                                  desc="The Country that the Last Port is located at")

    next_port_id = NumberField(index='NEXT_PORT_ID',
                               desc="A uniquely assigned ID by MarineTraffic for the Next Port")

    next_port_unlocode = TextField(index='NEXT_PORT_UNLOCODE',
                                   desc="A uniquely assigned ID by "
                                        "United Nations for the Next Port")

    next_port_name = TextField(index='NEXT_PORT_NAME',
                               desc="The Name of the Next Port as derived by MarineTraffic "
                                    "based on the subject vessel's reported Destination")

    next_port_country = TextField(index='NEXT_PORT_COUNTRY',
                                  desc="The Country that the Next Port is located at")

    eta_calc = DatetimeField(index='ETA_CALC',
                             desc="The Estimated Time of Arrival to Destination of "
                                  "the subject vessel according to the MarineTraffic calculations",
                             format='%Y-%m-%dT%H:%M:%S')

    eta_updated = DatetimeField(index='ETA_UPDATED',
                                desc="The date and time (in UTC) that the "
                                     "ETA was calculated by MarineTraffic",
                                format='%Y-%m-%dT%H:%M:%S')

    distance_to_go = NumberField(index='DISTANCE_TO_GO',
                                 desc="The Remaining Distance (in NM) for the subject "
                                      "vessel to reach the reported Destination")

    distance_travelled = NumberField(index='DISTANCE_TRAVELLED',
                                     desc="The Distance (in NM) that the subject "
                                          "vessel has travelled since departing from Last Port")

    awg_speed = RealNumberField(index='AVG_SPEED',
                                desc="The average speed calculated for the subject "
                                     "vessel during the latest voyage (port to port)")

    max_speed = RealNumberField(index='MAX_SPEED',
                                desc="The maximum speed reported by the subject "
                                     "vessel during the latest voyage (port to port)")



from typing import Any, Type, List, Union

from marinetrafficapi.formatter import Formatter, Json, Xml, Csv


class Response:
    """Response object returned from API call."""

    def __init__(self, data: Any, status_code: int,
                 formatter: Type[Formatter], api_reguest):
        self._data = data
        self._model = api_reguest.model

        self.api_reguest = api_reguest
        self.formatter = formatter()
        self.status_code = status_code

        self._response_data = {
            'to_list': None,
            'formatted': None,
            'models': []
        }

    @property
    def raw_data(self) -> Any:
        """Return data as is from the server."""

        return self._data

    @property
    def formatted_data(self) -> Union[Type[Json], Type[Xml],
                                      Type[Csv], None]:
        """Format data using secected formatter."""

        if self._response_data['formatted'] is None:
            self._response_data['formatted'] = \
                self.formatter.format(self._data)

        return self._response_data['formatted']

    @property
    def models(self) -> Union[List[object], None]:
        """Transform raw data into models."""

        if not self._response_data['models']:
            formatted_data = self.formatter.format(self._data)
            self._response_data['models'] = \
                self._model.process(
                    self.formatter.to_list(formatted_data))
            
        return self._response_data['models']

    @property
    def to_list(self) -> Union[List[object], None]:
        """Transform data into list."""

        if not self._response_data['to_list']:
            formatted_data = self.formatter.format(self._data)
            self._response_data['to_list'] = \
                self.formatter.to_list(formatted_data)

        return self._response_data['to_list']



from typing import List, Any

from marinetrafficapi.fields import Field


class Model:
    """Abstract model class."""

    def __init__(self, item: Any):
        self.item = item

    @classmethod
    def process(cls, data: list) -> List[object]:
        """Transform raw data into models."""

        model_list = []

        for item in data:
            model_object = cls(item)

            for model_property in cls.__dict__:
                property_object = getattr(model_object,
                                          model_property)

                if isinstance(property_object, Field):
                    property_object.convert_item(model_object)
                    setattr(model_object, model_property,
                            property_object.data)

            model_list.append(model_object)

        return model_list
